#!/bin/bash

# Make sure you are using bash instead of a Bourne shell.
# If you need to change the path to bash, you will also need to change it in
# mkmkfile.sh in directory src and all the directories under examples.

##############################################################################
#
# Instructions for modifying this file.
#
# Modification of this file for your system(s) consists of the following
# steps.  To find the places where these changes should be made,
# search for the string "Step"
#
# Step 1.  Set your default system configuration.
# Step 2.  Set paths, library names, etc.
# Step 3.  Reset paths, library names, etc. for specific system configurations
#
##############################################################################

##############################################################################

# Step 1.
# Set default system configuration.  These can be overridden by environment
# variables of the same name without "DEFAULT_".  Command line arguments
# of the same name without "DEFAULT_" override both these defaults and
# environment variables.  Command line arguments can also omit "PHAML_".
# The variable name and value should be separated by a space, e.g.
# "./mkmkfile.sh PARLIB mpi PHAML_F90 nag".
# Acceptable values are listed before each variable.  Hopefully they are
# self explanatory.  If not, look at the section on system specific settings.
# "./mkmkfile.sh help" will list all the variables, acceptable values and
# default values.
# You can also add your own values to the existing list if you think you
# know what you are doing.  Just use one of the existing values as a guide
# for what you need to add to this script.
# Naturally, not all combinations are valid.  There is very little checking
# of the validity of the combination.

# Architecture: mac origin rs6k sgi sun tflop x86
DEFAULT_PHAML_ARCH=x86

# OS: aix cougar cygwin irixn32 irix64 linux osx solaris
DEFAULT_PHAML_OS=linux

# Fortran 90 compiler: absoft g95 gfortran intel lahey mingw nag pathscale pgi sgi sun xlf
DEFAULT_PHAML_F90=intel

# C compiler: cc gcc
DEFAULT_PHAML_C=cc

# Hash size: 1 2
# (2 allows more levels of refinement, but uses more memory and time)
DEFAULT_PHAML_HASHSIZE=2

# Parallelism: messpass_spawn messpass_nospawn openmp hybrid_spawn hybrid_nospawn sequential
DEFAULT_PHAML_PARALLEL=messpass_spawn

# Parallel library: lam mpi mpich mpich2 myrinet openmpi none
# (mpi is used for native MPI libraries, as opposed to lam or mpich)
DEFAULT_PHAML_PARLIB=openmpi

# Element: triangle tetrahedron, tetrahedron_fixdeg
DEFAULT_PHAML_ELEMENT=triangle

# Graphics: metro mesa none opengl
DEFAULT_PHAML_GRAPHICS=mesa

# BLAS: atlas compiler goto petsc standard vendor
DEFAULT_PHAML_BLAS=standard

# LAPACK: atlas compiler petsc standard vendor
DEFAULT_PHAML_LAPACK=standard

# PETSc: no yes
DEFAULT_PHAML_PETSC=no

# SLEPc: no yes
DEFAULT_PHAML_SLEPC=no

# ARPACK: no yes
DEFAULT_PHAML_ARPACK=no

# BLOPEX: no yes
DEFAULT_PHAML_BLOPEX=no

# hypre: no yes
DEFAULT_PHAML_HYPRE=no

# ML: no yes
DEFAULT_PHAML_ML=no

# MUMPS: no yes
DEFAULT_PHAML_MUMPS=no

# SuperLU: no yes
DEFAULT_PHAML_SUPERLU=no

# Zoltan: no yes
DEFAULT_PHAML_ZOLTAN=no

# ParMETIS: no yes
DEFAULT_PHAML_PARMETIS=no

# DRUM: no yes
DEFAULT_PHAML_DRUM=no

# Specific system: none dirac looney pepe raritan suns stampede
DEFAULT_PHAML_SYSTEM=none

# end of Step 1; proceed to Step 2 below

##############################################################################

# Set the system configuration from environment variables when they are
# set, and the default when not.

if [ -z $PHAML_ARCH ]
then
   PHAML_ARCH=$DEFAULT_PHAML_ARCH
fi
if [ -z $PHAML_OS ]
then
   PHAML_OS=$DEFAULT_PHAML_OS
fi
if [ -z $PHAML_F90 ]
then
   PHAML_F90=$DEFAULT_PHAML_F90
fi
if [ -z $PHAML_C ]
then
   PHAML_C=$DEFAULT_PHAML_C
fi
if [ -z $PHAML_HASHSIZE ]
then
   PHAML_HASHSIZE=$DEFAULT_PHAML_HASHSIZE
fi
if [ -z $PHAML_PARALLEL ]
then
   PHAML_PARALLEL=$DEFAULT_PHAML_PARALLEL
fi
if [ -z $PHAML_PARLIB ]
then
   PHAML_PARLIB=$DEFAULT_PHAML_PARLIB
fi
if [ -z $PHAML_ELEMENT ]
then
   PHAML_ELEMENT=$DEFAULT_PHAML_ELEMENT
fi
if [ -z $PHAML_GRAPHICS ]
then
   PHAML_GRAPHICS=$DEFAULT_PHAML_GRAPHICS
fi
if [ -z $PHAML_BLAS ]
then
   PHAML_BLAS=$DEFAULT_PHAML_BLAS
fi
if [ -z $PHAML_LAPACK ]
then
   PHAML_LAPACK=$DEFAULT_PHAML_LAPACK
fi
if [ -z $PHAML_PETSC ]
then
   PHAML_PETSC=$DEFAULT_PHAML_PETSC
fi
if [ -z $PHAML_SLEPC ]
then
   PHAML_SLEPC=$DEFAULT_PHAML_SLEPC
fi
if [ -z $PHAML_ARPACK ]
then
   PHAML_ARPACK=$DEFAULT_PHAML_ARPACK
fi
if [ -z $PHAML_BLOPEX ]
then
   PHAML_BLOPEX=$DEFAULT_PHAML_BLOPEX
fi
if [ -z $PHAML_HYPRE ]
then
   PHAML_HYPRE=$DEFAULT_PHAML_HYPRE
fi
if [ -z $PHAML_ML ]
then
   PHAML_ML=$DEFAULT_PHAML_ML
fi
if [ -z $PHAML_MUMPS ]
then
   PHAML_MUMPS=$DEFAULT_PHAML_MUMPS
fi
if [ -z $PHAML_SUPERLU ]
then
   PHAML_SUPERLU=$DEFAULT_PHAML_SUPERLU
fi
if [ -z $PHAML_ZOLTAN ]
then
   PHAML_ZOLTAN=$DEFAULT_PHAML_ZOLTAN
fi
if [ -z $PHAML_PARMETIS ]
then
   PHAML_PARMETIS=$DEFAULT_PHAML_PARMETIS
fi
if [ -z $PHAML_DRUM ]
then
   PHAML_DRUM=$DEFAULT_PHAML_DRUM
fi
if [ -z $PHAML_SYSTEM ]
then
   PHAML_SYSTEM=$DEFAULT_PHAML_SYSTEM
fi

##############################################################################

# Change system configuration based on command line arguments

while [ $# -gt 0 ]
do
   if [ "$1" = "help" ]
   then
      echo "mkmkfile.sh [ VAR val]* where VAR, choices for val, and (current setting) are:"
      echo "  ARCH mac origin rs6k sgi sun tflop x86 ($PHAML_ARCH)"
      echo "  OS aix cougar cygwin irixn32 irix64 linux osx solaris ($PHAML_OS)"
      echo "  F90 absoft g95 gfortran intel lahey mingw nag pathscale pgi sgi sun xlf ($PHAML_F90)"
      echo "  C cc gcc ($PHAML_C)"
      echo "  HASHSIZE 1 2 ($PHAML_HASHSIZE)"
      echo "  PARALLEL messpass_spawn messpass_nospawn openmp hybrid_spawn hybrid_nospawn sequential ($PHAML_PARALLEL)"
      echo "  PARLIB lam mpi mpich mpich2 myrinet openmpi none ($PHAML_PARLIB)"
      echo "  ELEMENT triangle tetrahedron tetrahedron_fixdeg ($PHAML_ELEMENT)"
      echo "  GRAPHICS metro mesa none opengl ($PHAML_GRAPHICS)"
      echo "  BLAS atlas compiler goto petsc standard vendor ($PHAML_BLAS)"
      echo "  LAPACK atlas compiler petsc standard vendor ($PHAML_LAPACK)"
      echo "  SLEPC no yes ($PHAML_SLEPC)"
      echo "  ARPACK no yes ($PHAML_ARPACK)"
      echo "  BLOPEX no yes ($PHAML_BLOPEX)"
      echo "  PETSC no yes ($PHAML_PETSC)"
      echo "  MUMPS no yes ($PHAML_MUMPS)"
      echo "  SUPERLU no yes ($PHAML_SUPERLU)"
      echo "  HYPRE no yes ($PHAML_HYPRE)"
      echo "  ML no yes ($PHAML_ML)"
      echo "  ZOLTAN no yes ($PHAML_ZOLTAN)"
      echo "  PARMETIS no yes ($PHAML_PARMETIS)"
      echo "  DRUM no yes ($PHAML_DRUM)"
      echo "  SYSTEM none dirac looney pepe raritan suns stampede ($PHAML_SYSTEM)"
      exit 0
   fi
   if [ $# -eq 1 ]
   then
      echo "mkmkfile.sh: value for configuration variable $1 is missing"
      exit 1
   else
      val=$2
   fi
   case "$1" in
      PHAML_ARCH|ARCH) PHAML_ARCH=$val ;;
      PHAML_OS|OS) PHAML_OS=$val ;;
      PHAML_F90|F90) PHAML_F90=$val ;;
      PHAML_C|C) PHAML_C=$val ;;
      PHAML_HASHSIZE|HASHSIZE) PHAML_HASHSIZE=$val ;;
      PHAML_PARALLEL|PARALLEL) PHAML_PARALLEL=$val ;;
      PHAML_PARLIB|PARLIB) PHAML_PARLIB=$val ;;
      PHAML_ELEMENT|ELEMENT) PHAML_ELEMENT=$val ;;
      PHAML_GRAPHICS|GRAPHICS) PHAML_GRAPHICS=$val ;;
      PHAML_BLAS|BLAS) PHAML_BLAS=$val ;;
      PHAML_LAPACK|LAPACK) PHAML_LAPACK=$val ;;
      PHAML_SLEPC|SLEPC) PHAML_SLEPC=$val ;;
      PHAML_ARPACK|ARPACK) PHAML_ARPACK=$val ;;
      PHAML_BLOPEX|BLOPEX) PHAML_BLOPEX=$val ;;
      PHAML_PETSC|PETSC) PHAML_PETSC=$val ;;
      PHAML_MUMPS|MUMPS) PHAML_MUMPS=$val ;;
      PHAML_SUPERLU|SUPERLU) PHAML_SUPERLU=$val ;;
      PHAML_HYPRE|HYPRE) PHAML_HYPRE=$val ;;
      PHAML_ML|ML) PHAML_ML=$val ;;
      PHAML_ZOLTAN|ZOLTAN) PHAML_ZOLTAN=$val ;;
      PHAML_PARMETIS|PARMETIS) PHAML_PARMETIS=$val ;;
      PHAML_DRUM|DRUM) PHAML_DRUM=$val ;;
      PHAML_SYSTEM|SYSTEM) PHAML_SYSTEM=$val ;;
      *) echo "mkmkfile.sh: argument $1 is not a system configuration variable"
         exit 1 ;;
   esac
   shift
   shift
done

##############################################################################

# Check validity of system configuration

# Check that values are in the recognized lists

case "$PHAML_ARCH" in
   mac|origin|rs6k|sgi|sun|tflop|x86) ;;
   *) echo "mkmkfile.sh: $PHAML_ARCH is not a valid value for PHAML_ARCH"
      echo "             use one of mac origin rs6k sgi sun tflop x86"
      exit 1 ;;
esac
case "$PHAML_OS" in
   aix|cougar|cygwin|irixn32|irix64|linux|osx|solaris) ;;
   *) echo "mkmkfile.sh: $PHAML_OS is not a valid value for PHAML_OS"
      echo "             use one of aix cougar irixn32 irix64 linux osx solaris"
      exit 1 ;;
esac
case "$PHAML_F90" in
   absoft|g95|gfortran|intel|lahey|mingw|nag|pathscale|pgi|sgi|sun|xlf) ;;
   *) echo "mkmkfile.sh: $PHAML_F90 is not a valid value for PHAML_F90"
      echo "             use one of absoft g95 gfortran intel lahey nag pathscale pgi sgi sun xlf"
      exit 1 ;;
esac
case "$PHAML_C" in
   cc|gcc) ;;
   *) echo "mkmkfile.sh: $PHAML_C is not a valid value for PHAML_C"
      echo "             use one of cc gcc"
      exit 1 ;;
esac
case "$PHAML_HASHSIZE" in
   1|2) ;;
   *) echo "mkmkfile.sh: $PHAML_HASHSIZE is not a valid value for PHAML_HASHSIZE"
      echo "             use one of 1 2"
      exit 1 ;;
esac
case "$PHAML_PARALLEL" in
   messpass_spawn|messpass_nospawn|openmp|hybrid_spawn|hybrid_nospawn|sequential) ;;
   *) echo "mkmkfile.sh: $PHAML_PARALLEL is not a valid value for PHAML_PARALLEL"
      echo "             use one of messpass_spawn messpass_nospawn openmp hybrid_spawn hybrid_nospawn sequential"
      exit 1 ;;
esac
case "$PHAML_PARLIB" in
   lam|mpi|mpich|mpich2|myrinet|openmpi|none) ;;
   *) echo "mkmkfile.sh: $PHAML_PARLIB is not a valid value for PHAML_PARLIB"
      echo "             use one of lam mpi mpich mpich2 myrinet openmpi none"
      exit 1 ;;
esac
case "$PHAML_ELEMENT" in
   triangle|tetrahedron|tetrahedron_fixdeg) ;;
   *) echo "mkmkfile.sh: $PHAML_ELEMENT is not a valid value for PHAML_ELEMENT"
      echo "             use one of triangle tetrahedron tetrahedron_fixdeg"
      exit 1 ;;
esac
case "$PHAML_GRAPHICS" in
   metro|mesa|none|opengl) ;;
   *) echo "mkmkfile.sh: $PHAML_GRAPHICS is not a valid value for PHAML_GRAPHICS"
      echo "             use one of metro mesa none opengl"
      exit 1 ;;
esac
case "$PHAML_BLAS" in
   atlas|compiler|goto|petsc|standard|vendor) ;;
   *) echo "mkmkfile.sh: $PHAML_BLAS is not a valid value for PHAML_BLAS"
      echo "             use one of atlas compiler goto standard vendor"
      exit 1 ;;
esac
case "$PHAML_LAPACK" in
   atlas|compiler|petsc|standard|vendor) ;;
   *) echo "mkmkfile.sh: $PHAML_LAPACK is not a valid value for PHAML_LAPACK"
      echo "             use one of atlas compiler standard vendor"
      exit 1 ;;
esac
case "$PHAML_SLEPC" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_SLEPC is not a valid value for PHAML_SLEPC"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_ARPACK" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_ARPACK is not a valid value for PHAML_ARPACK"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_BLOPEX" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_BLOPEX is not a valid value for PHAML_BLOPEX"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_PETSC" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_PETSC is not a valid value for PHAML_PETSC"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_MUMPS" in
   no|yes|standalone) ;;
   *) echo "mkmkfile.sh: $PHAML_MUMPS is not a valid value for PHAML_MUMPS"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_SUPERLU" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_SUPERLU is not a valid value for PHAML_SUPERLU"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_HYPRE" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_HYPRE is not a valid value for PHAML_HYPRE"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_ML" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_ML is not a valid value for PHAML_ML"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_ZOLTAN" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_ZOLTAN is not a valid value for PHAML_ZOLTAN"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_PARMETIS" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_PARMETIS is not a valid value for PHAML_PARMETIS"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_DRUM" in
   no|yes) ;;
   *) echo "mkmkfile.sh: $PHAML_DRUM is not a valid value for PHAML_DRUM"
      echo "             use one of no yes"
      exit 1 ;;
esac
case "$PHAML_SYSTEM" in
   none|dirac|looney|pepe|raritan|suns|stampede) ;;
   *) echo "mkmkfile.sh: $PHAML_SYSTEM is not a valid value for PHAML_SYSTEM"
      echo "             use one of none dirac looney pepe raritan suns stampede"
      exit 1 ;;
esac

# set some values based on dependencies

if [ $PHAML_PARALLEL == "sequential" ]
then
   if [ $PHAML_PARLIB != "none" ]
   then
      echo "mkmkfile.sh WARNING: setting PARLIB to none because PARALLEL is sequential"
      PHAML_PARLIB=none
   fi
   if [ $PHAML_HYPRE != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting HYPRE to no because PARALLEL is sequential"
      PHAML_HYPRE=no
   fi
   if [ $PHAML_ML != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting ML to no because PARALLEL is sequential"
      PHAML_ML=no
   fi
   if [ $PHAML_ZOLTAN != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting ZOLTAN to no because PARALLEL is sequential"
      PHAML_ZOLTAN=no
   fi
fi

if [ $PHAML_PARALLEL == "openmp" ]
then
   if [ $PHAML_PARLIB != "none" ]
   then
      echo "mkmkfile.sh WARNING: setting PARLIB to none because PARALLEL is openmp"
      PHAML_PARLIB=none
   fi
   if [ $PHAML_HYPRE != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting HYPRE to no because PARALLEL is openmp"
      PHAML_HYPRE=no
   fi
   if [ $PHAML_ML != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting ML to no because PARALLEL is openmp"
      PHAML_ML=no
   fi
   if [ $PHAML_ZOLTAN != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting ZOLTAN to no because PARALLEL is openmp"
      PHAML_ZOLTAN=no
   fi
fi

if [ $PHAML_PETSC == "no" ]
then
   if [ $PHAML_SLEPC != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting SLEPC to no because PETSC is no"
      PHAML_SLEPC=no
   fi
   if [ $PHAML_HYPRE != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting HYPRE to no because PETSC is no"
      PHAML_HYPRE=no
   fi
   if [ $PHAML_ML != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting ML to no because PETSC is no"
      PHAML_ML=no
   fi
   if [ $PHAML_MUMPS != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting MUMPS to no because PETSC is no"
      PHAML_MUMPS=no
   fi
   if [ $PHAML_SUPERLU != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting SUPERLU to no because PETSC is no"
      PHAML_SUPERLU=no
   fi
   if [ $PHAML_BLAS == "petsc" ]
   then
      echo "mkmkfile.sh WARNING: setting BLAS to standard because PETSC is no"
      PHAML_BLAS=standard
   fi
   if [ $PHAML_LAPACK == "petsc" ]
   then
      echo "mkmkfile.sh WARNING: setting LAPACK to standard because PETSC is no"
      PHAML_LAPACK=standard
   fi
fi

if [ $PHAML_SLEPC == "no" ]
then
   if [ $PHAML_ARPACK != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting ARPACK to no because SLEPC is no"
      PHAML_ARPACK=no
   fi
   if [ $PHAML_BLOPEX != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting BLOPEX to no because SLEPC is no"
      PHAML_BLOPEX=no
   fi
fi

if [ $PHAML_ZOLTAN == "no" ]
then
   if [ $PHAML_PARMETIS != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting PARMETIS to no because ZOLTAN is no"
      PHAML_PARMETIS=no
   fi
   if [ $PHAML_DRUM != "no" ]
   then
      echo "mkmkfile.sh WARNING: setting DRUM to no because ZOLTAN is no"
      PHAML_DRUM=no
   fi
fi

# check for easy-to-identify incorrect combinations

if [ $PHAML_PARALLEL == "sequential" -a $PHAML_PARLIB != "none" ]
then
   echo "mkmkfile.sh: PARALLEL==sequential requires PARLIB==none"
   exit 1
fi

if [ $PHAML_PARALLEL == "openmp" -a $PHAML_PARLIB != "none" ]
then
   echo "mkmkfile.sh: PARALLEL==openmp requires PARLIB==none"
   exit 1
fi

if [ $PHAML_PARALLEL == "hybrid_spawn" -a $PHAML_PARLIB == "lam" ]
then
   echo "mkmkfile.sh: LAM and OpenMP do not work together"
   exit 1
fi

if [ $PHAML_PARALLEL == "hybrid_nospawn" -a $PHAML_PARLIB == "lam" ]
then
   echo "mkmkfile.sh: LAM and OpenMP do not work together"
   exit 1
fi

if [ $PHAML_PARALLEL != "sequential" -a $PHAML_PARALLEL != "openmp" -a $PHAML_PARLIB == "none" ]
then
   echo "mkmkfile.sh: if PARALLEL is not sequential or openmp then PARLIB cannot be none"
   exit 1
fi

if [ $PHAML_PARLIB == "mpich" -a $PHAML_PARALLEL != "messpass_nospawn" ]
then
   echo "mkmkfile.sh: PARLIB==mpich requires PARALLEL==messpass_nospawn"
   exit 1
fi

if [ $PHAML_PARLIB != "lam" -a $PHAML_PARLIB != "mpi" -a $PHAML_PARLIB != "mpich" -a $PHAML_PARLIB != "mpich2" -a $PHAML_PARLIB != "myrinet" -a $PHAML_PARLIB != "openmpi" ]
then
   if [ $PHAML_ZOLTAN != "no" ]
   then
      echo "mkmkfile.sh: Zoltan requires an MPI library"
      exit 1
   fi
fi

if [ $PHAML_SLEPC == "yes" -a $PHAML_PETSC == "no" ]
then
   echo "mkmkfile.sh: SLEPC==yes requires PETSC==yes"
   exit 1
fi

if [ $PHAML_ZOLTAN == "no" -a $PHAML_PARMETIS == "yes" ]
then
   echo "mkmkfile.sh: PARMETIS==yes requires ZOLTAN==yes"
   exit 1
fi

if [ $PHAML_ZOLTAN == "no" -a $PHAML_DRUM == "yes" ]
then
   echo "mkmkfile.sh: DRUM==yes requires ZOLTAN==yes"
   exit 1
fi

##############################################################################

# Misc variables

if [ $PHAML_F90 = "mingw" ]
then
   MAKELIB="x86_64-w64-mingw32-ar rcv"
   RANLIB=x86_64-w64-mingw32-ranlib
elif [ $PHAML_OS = "osx" ]
then
   MAKELIB="libtool -static -no_warning_for_no_symbols -o"
   RANLIB="do not use"
else
   MAKELIB="ar rcv"
   RANLIB=ranlib
fi
PHAML_HOME=`pwd`

##############################################################################

# Step 2.  Set paths, library names, etc.
# In this section, set the variables that specify the specific names of
# libraries, paths to libraries, paths to module and include files,
# command names, etc.  Of course, you only need to set those you are
# actually going to use!  Most of these variables are used directly on the
# command line, so usually should include whatever compiler flags
# accompany them.  Follow the form used in the examples.  Some of these may
# appear to be unneccessarily complicated; this is because our machines have
# multiple compilers and libraries installed which would conflict if not
# handled carefully.  Your paths are likely to be much simpler, or even
# nonexistent in many cases (where the system already knows it).  Note the use
# of single quotes when a dollar sign is to be passed through.
# Values set here can be overwritten by step 3 if different system
# configurations require different values.
# Specific version numbers quoted here indicate the last version of a
# software package that PHAML was tested with.  This is not to imply
# that earlier or later versions will not work.

case "$PHAML_F90" in

# Absoft 64-bit Pro Fortran 13.0.3

   absoft)
      F90=af95
# starting with Version 10.1 Absoft uses the lower case and attach underscore
# name mangling.  Earlier versions need -YEXT_NAMES=LCS -YEXT_SFX=_ -YCFRL=1
      FFLAGS="-O -w -YNO_CDEC"
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            echo "mkmkfile.sh: As of this writing the Absoft compiler does not support OpenMP."
            echo "If this has changed, modify mkmkfile.sh."
            exit 1 ;;
      esac
      MODFLAG="-p "
      MODSUFFIX=mod ;;

# g95 Stable version 0.92, June 2009

   g95)
      F90=g95
      FFLAGS="-O -fno-second-underscore"
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            echo "mkmkfile.sh: As of this writing g95 does not support OpenMP."
            echo "If this has changed, modify mkmkfile.sh."
            exit 1 ;;
      esac
      MODFLAG=-I
      MODSUFFIX=mod ;;

# GNU Fortran (GCC) 4.8.5 20150623 (Red Hat 4.8.5-4)

   gfortran)
      F90=gfortran
      FFLAGS="-O -w"
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -fopenmp"
            LINKFLAGS="$LINKFLAGS -fopenmp" ;;
      esac
      MODFLAG="-p "
      MODFLAG=-I
      MODSUFFIX=mod ;;

# MinGW for building a dll

   mingw)
      F90=x86_64-w64-mingw32-gfortran
      FFLAGS="-O -fno-second-underscore"
      LINKER=$F90
      LINKFLAGS="-static-libgcc -static-libgfortran -Wl,-allow-multiple-definition -fstack-protector"
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -fopenmp"
            LINKFLAGS="$LINKFLAGS -fopenmp" ;;
      esac
      MODFLAG="-p "
      MODFLAG=-I
      MODSUFFIX=mod ;;

# Intel(R) Parallel Studio XE Composer Edition for Fortran Linux 2018
# ifort (IFORT) 18.0.3 20180410

   intel)
      F90=ifort
      FFLAGS="-O -w"
      LINKER=$F90
      LINKFLAGS=-w
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -qopenmp"
            LINKFLAGS="$LINKFLAGS -qopenmp" ;;
      esac
      MODFLAG="-p "
      MODFLAG=-I
      MODSUFFIX=mod ;;

# Lahey lf95 L8.10b

   lahey)
      F90=lf95
      FFLAGS=-O
      LINKER=$F90
      LINKFLAGS=--staticlink
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS --openmp"
            LINKFLAGS="$LINKFLAGS --openmp" ;;
      esac
      MODFLAG=-I
      MODSUFFIX=mod ;;

# NAG Fortran Compiler Release 6.2(Chiyoda) Build 6202

   nag)
      F90=nagfor
      FFLAGS='-O -w -mismatch -ieee=full'
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -openmp"
            LINKFLAGS="$LINKFLAGS -openmp" ;;
      esac
      MODFLAG=-I
      MODSUFFIX=mod ;;

# PathScale EKOPath Fortran 95 Version 2.1, sold by Lahey

   pathscale)
      F90=pathf90
      FFLAGS=-O
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -openmp"
            LINKFLAGS="$LINKFLAGS -openmp" ;;
      esac
      MODFLAG=-I
      MODSUFFIX=mod ;;

# PGI pgf90 6.0-2

   pgi)
      F90=pgf90
      FFLAGS=-O
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -mp"
            LINKFLAGS="$LINKFLAGS -mp" ;;
      esac
      MODFLAG=-I
      MODFLAG=-I
      MODSUFFIX=mod ;;

# SGI MIPSpro f90 7.4.4m

   sgi)
      F90=f90
      case "$PHAML_OS" in
         irixn32)
            FFLAGS="-O -n32"
            LINKFLAGS=-n32 ;;
         irix64)
            FFLAGS="-O -64"
            LINKFLAGS=-64 ;;
      esac
      LINKER=$F90
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -mp"
            LINKFLAGS="$LINKFLAGS -mp" ;;
      esac
      MODFLAG=-I
      MODSUFFIX=mod ;;

# Sun Forte Developer 7 Fortran 95 7.0 2002/03/09

   sun)
      F90=f90
# This compiler crashes while compiling grid.f90 if -O is used
      FFLAGS=
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            FFLAGS="$FFLAGS -openmp"
            LINKFLAGS="$LINKFLAGS -openmp" ;;
      esac
      MODFLAG=-M
      MODSUFFIX=mod ;;

# IBM XLF90 4.1.0.6

   xlf)
      F90=xlf90
      FFLAGS="-O -F:f90"
      LINKER=$F90
      LINKFLAGS=
      case "$PHAML_PARALLEL" in
         openmp|hybrid_spawn|hybrid_nospawn)
            F90=xlf90_r
            LINKER=$F90
            FFLAGS="$FFLAGS -qsmp=omp"
            LINKFLAGS="LINKFLAGS -qsmp=omp" ;;
      esac
      MODFLAG=-I
      MODSUFFIX=mod ;;

# invalid value for PHAML_F90

   *) echo "mkmkfile.sh: $PHAML_F90 is not a valid value for PHAML_F90"
      exit 1 ;;

esac

case "$PHAML_C" in

# native C compiler

   cc)
      CC=cc
      case "$PHAML_OS" in
         irix64)
            CFLAGS="-O -64" ;;
         *)
            CFLAGS=-O ;;
      esac ;;

# gcc (GCC) 4.8.5 20150623 (Red Hat 4.8.5-16)

   gcc)
      if [ $PHAML_F90 = "mingw" ]
      then
         CC=x86_64-w64-mingw32-gcc
      else
         CC=gcc
      fi
      CFLAGS=-O ;;

# invalid value for PHAML_C

   *) echo "mkmkfile.sh: $PHAML_C is not a valid value for PHAML_C"
      exit 1 ;;

esac

case "$PHAML_PARLIB" in

# LAM 7.1.4

   lam)
      RUNMPI="mpirun"
      MESSPASSINC=
      MESSPASSLIBS=
      MESSPASSPACK='mpipack1.o mpipack2.o'
      MPIF=$F90
      F90="mpif77"
      LINKER="mpif77"
      CC=mpicc ;;

# native MPI libraries; probably want to override in the system specific section

   mpi)
      RUNMPI="mpiexec"
      MESSPASSINC=""
      MESSPASSLIBS="-lmpi"
      MESSPASSPACK='mpipack1.o mpipack2.o' ;;

# MPICH 1.2.7

   mpich)
      RUNMPI="mpirun"
#      MESSPASSINC='-I$(MPICH_HOME)'"/$PHAML_F90/include"
#      MESSPASSLIBS='-L$(MPICH_HOME)'"/$PHAML_F90/lib -lmpich"
      MESSPASSINC=
      MESSPASSLIBS=
      MESSPASSPACK='mpipack1.o mpipack2.o' ;;

# MPICH2 1.1.1p1

   mpich2)
      RUNMPI="mpiexec"
      MESSPASSINC=
      MESSPASSLIBS=
      MESSPASSPACK='mpipack1.o mpipack2.o'
      MPIF=$F90
      F90="mpif90"
      LINKER="mpif90"
      CC=mpicc ;;

# Myrinet MPICH 1.2.4

   myrinet)
      RUNMPI="mpirun"
      MESSPASSINC='-I$(MPICH_HOME)/include'
      MESSPASSLIBS='-L$(MPICH_HOME)/lib -lpmpich -lmpich -L/home/pozo/MYRICOM/gm-1.2.3/binary/lib -lgm'
      MESSPASSPACK='mpipack1.o mpipack2.o' ;;

# Open MPI openmpi-1.10.7-1.el7.x86_64

   openmpi)

      RUNMPI="mpirun"
      MESSPASSINC=
      MESSPASSLIBS=
      MESSPASSPACK='mpipack1.o mpipack2.o'
      MPIF=$F90
      F90="mpifort"
      LINKER="mpifort"
      CC=mpicc ;;

# no message passing library

   none)
      RUNMPI="./"
      MESSPASSINC=""
      MESSPASSLIBS=""
      MESSPASSPACK="" ;;

# invalid value for PHAML_PARLIB

   *) echo "mkmkfile.sh: $PHAML_PARLIB is not a valid value for PHAML_PARLIB"
      exit 1 ;;

esac

# X11R6

XINCL=-I/usr/X11R6/include
XLIBS="-L/usr/X11R6/lib -lXaw -lXt -lXmu -lXi -lX11 -lXext -lm"

case "$PHAML_GRAPHICS" in

# MetroLink OpenGL 1.3, Glut 3.7.1, f90gl 1.2.0

   metro)
      OGLMODS="$MODFLAG"'$(OPENGL_HOME)/include/GL'
      OGLLIBS='-L$(OPENGL_HOME)/lib -L/usr/X11R6/lib -lf90glut -lf90GLU -lf90GL -lglut -lGLU -lGL' ;;

# Mesa 7.8.2, Glut MesaGLUT 7.8.2, f90gl 1.2.15

   mesa)
      OGLMODS="$MODFLAG"'$(MESA_HOME)/'"$PHAML_F90/include/GL"
      OGLLIBS='-L$(MESA_HOME)/lib -L$(MESA_HOME)/'"$PHAML_F90/lib -lf90glut -lf90GLU -lf90GL -lglut -lGLU -lGL" ;;

# No graphics library

   none)
      OGLMODS=
      OGLLIBS= ;;

# Native OpenGL, may need to redefine in the system specific section

   opengl)
      OGLMODS="$MODFLAG"'$(OPENGL_HOME)/include/GL'
      OGLLIBS='-L$(OPENGL_HOME)/lib -L/usr/X11R6/lib -lf90glut -lf90GLU -lf90GL -lglut -lGLU -lGL' ;;

# invalid value for PHAML_GRAPHICS

   *) echo "mkmkfile.sh: $PHAML_GRAPHICS is not a valid value for PHAML_GRAPHICS"
      exit 1 ;;

esac

# Zoltan 3.3

# As of version 3.2 (or maybe 3.1) only one library is needed.  Select the
# correct form.

#ZOLTANLIB='-L$(ZOLTAN_HOME)'"/$PHAML_F90/$PHAML_PARLIB/lib -lzoltan -lzoltan_comm -lzoltan_mem"
ZOLTANLIB='-L$(ZOLTAN_HOME)'"/$PHAML_F90/$PHAML_PARLIB/lib -lzoltan"

ZOLTANMOD="$MODFLAG"'$(ZOLTAN_HOME)'"/$PHAML_F90/$PHAML_PARLIB/include"
ZOLTANINC='-I$(ZOLTAN_HOME)/include -I$(ZOLTAN_HOME)/Utilities/Communication -I$(ZOLTAN_HOME)/Utilities/Memory -I$(ZOLTAN_HOME)/Utilities/DDirectory'

# ParMETIS 3.1 from Zoltan 3.3

PARMETISLIB='-L$(PARMETIS_HOME)'"/$PHAML_PARLIB/lib -lparmetis -lmetis"

# DRUM 2.0

DRUMLIB='-L$(DRUM_HOME)/lib/speedy -ldrum -lrt -lxml2'

case "$PHAML_ARPACK" in

# ARPACK no version number

   yes)
      if [ $PHAML_PARALLEL = "sequential" -o $PHAML_PARALLEL = "openmp" ]
      then
         ARPACKLIBS='-L$(ARPACK_HOME)/'"$PHAML_F90/$PHAML_PARLIB/lib -larpack"
      else
         ARPACKLIBS='-L$(ARPACK_HOME)/'"$PHAML_F90/$PHAML_PARLIB/lib -lparpack -larpack"
      fi ;;

   no)
      ARPACKLIBS= ;;

# invalid value for PHAML_ARPACK

   *) echo "mkmkfile.sh: $PHAML_ARPACK is not a valid value for PHAML_ARPACK"
      exit 1 ;;

esac

case "$PHAML_BLAS" in

# ATLAS 3.6.0

   atlas)
      BLASLIBS='-L$(ATLAS_HOME) -lf77blas -lcblas -latlas' ;;

# some compilers come with prebuilt BLAS libraries

   compiler)
      case "$PHAML_F90" in
         pgi)
            BLASLIBS='-L$(PGI_HOME)/lib -lblas' ;;
         lahey)
            BLASLIBS='-L$(LAHEY_HOME)/lib -lblasmt' ;;
         intel)
# or use mkl=sequential for nonthreaded library
            LINKFLAGS="$LINKFLAGS -mkl"
            BLASLIBS='' ;;
         *)
            BLASLIBS='-L/usr/lib -lblas' ;;
      esac ;;

# use BLAS built with PETSc using --download-f-blas-lapack

   petsc)
      BLASLIBS='-L$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/lib -lfblas' ;;

   goto)
      BLASLIBS='-L$(GOTO_HOME) -lgoto -lpthread' ;;

# use the standard BLAS library for this system

   standard)
      BLASLIBS='-L/usr/lib -lblas' ;;

# some hardware vendors supply BLAS libraries

   vendor)
      case "$PHAML_ARCH" in
         x86)
# MKL 18.2.046 supports intel, gfortran and absoft; otherwise use standard
            if [ $PHAML_F90 = "intel" ]
            then
               BLASLIBS='-L$(MKL_HOME) -lmkl_intel_lp64 -lmkl_intel_thread -lmkl_core -liomp5'
            else
               if [ $PHAML_F90 = "gfortran" -o $PHAML_F90 = "absoft" ]
               then
                  BLASLIBS='-L$(MKL_HOME) -lmkl_gf_lp64 -lmkl_sequential -lmkl_core'
               else
                  BLASLIBS='-L/usr/lib -lblas'
               fi
            fi ;;

         sgi)
            BLASLIBS='-lscs' ;;
         sun)
            BLASLIBS='-xlic_lib=sunperf' ;;
         *)
            BLASLIBS='-L/usr/lib -lblas' ;;
      esac ;;

# invalid value for PHAML_BLAS

   *) echo "mkmkfile.sh: $PHAML_BLAS is not a valid value for PHAML_BLAS"
      exit 1 ;;

esac

case "$PHAML_LAPACK" in

# ATLAS 3.6.0

   atlas)
      LAPACKLIBS='-L$(ATLAS_HOME) -llapack' ;;

# some compilers come with prebuilt LAPACK libraries

   compiler)
      case "$PHAML_F90" in
         pgi)
            LAPACKLIBS='-L$(PGI_HOME)/lib -llapack' ;;
         lahey)
            LAPACKLIBS='-L$(LAHEY_HOME)/lib -llapackmt' ;;
         intel)
# Don't need them if BLAS already put them in, but need something so it
# doesn't compile lapack.f
            if [ $PHAML_BLAS = "compiler" -o $PHAML_BLAS = "vendor" ]
            then
               LAPACKLIBS='-lpthread'
            else
               LINKFLAGS="$LINKFLAGS -mkl"
               LAPACKLIBS='-lpthread'
            fi ;;
         *)
            LAPACKLIBS='-L/usr/lib -llapack' ;;
      esac ;;

# use LAPACK built with PETSc using --download-f-blas-lapack

   petsc)
      LAPACKLIBS='-L$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/lib -lflapack' ;;

# use the standard LAPACK library for this system

   standard)
      LAPACKLIBS='-L/usr/lib -llapack' ;;

# some hardware vendors provide LAPACK libraries

   vendor)
      case "$PHAML_ARCH" in
         x86)
            if [ $PHAML_BLAS = "compiler" -o $PHAML_BLAS = "vendor" ]
            then
               LAPACKLIBS='-lpthread'
            else
               if [ $PHAML_F90 = "intel" ]
               then
                  LAPACKLIBS='-L$(MKL_HOME) -lmkl_intel_lp64 -lmkl_intel_thread -lmkl_core -liomp5'
               else
                  if [ $PHAML_F90 = "gfortran" -o $PHAML_F90 = "absoft" ]
                  then
                     LAPACKLIBS='-L$(MKL_HOME) -lmkl_gf_lp64 -lmkl_sequential -lmkl_core'
                  else
                     LAPACKLIBS=
                  fi
               fi
            fi ;;
         sgi)
            LAPACKLIBS='-lscs' ;;
         sun)
            LAPACKLIBS='-xlic_lib=sunperf' ;;
         *)
            LAPACKLIBS='-L/usr/lib -llapack' ;;
      esac ;;

# invalid value for PHAML_LAPACK

   *) echo "mkmkfile.sh: $PHAML_LAPACK is not a valid value for PHAML_LAPACK"
      exit 1 ;;

esac

# PETSc 3.9.1 (Versions before 2.3.1 may need changes in petsc_init.F90 and
#              petsc_interf.F90.  Search for "before" to find them.  Also,
#              select the right one for PETSCLIBS right here.)

case "$PHAML_PETSC" in

   yes)
# PETSc versions before 2.3.1
#      PETSCLIBS='-L$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/lib/$(PETSC_ARCH) -lpetscfortran -lpetscksp -lpetscts -lpetscsnes -lpetscdm -lpetscmat -lpetscvec -lpetsc'

# PETSc version 2.3.1 through 3.0
#      PETSCLIBS='-L$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/lib -lpetscksp -lpetscts -lpetscsnes -lpetscdm -lpetscmat -lpetscvec -lpetsc'

# PETSc version 3.1 and later allow compiling a single library.  If you did
# that, then you'll want this version
#      PETSCLIBS='-L$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/lib -lpetsc'

# With PETSc version 3.5.2 I get undefined references for things in libssl,
# so add library.  I don't know if this is a temporary fluke, an intentional
# change or something wrong in my setup, but google didn't show me anyone else
# complaining about it.
# With PETSc version 3.7.3 it is missing something in libcrypto.
      PETSCLIBS="-L$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB"'/lib -lpetsc -lssl -lcrypto'
# The NAG compiler also needs libdl
      if [ $PHAML_F90 = "nag" ]
      then
      PETSCLIBS="-L$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB"'/lib -lpetsc -lssl -lcrypto -ldl'
      fi
# gfortran also needs libdl and libstdc++
      if [ $PHAML_F90 = "gfortran" ]
      then
      PETSCLIBS="-L$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB"'/lib -lpetsc -lssl -lcrypto -ldl -lstdc++'
      fi
# cygwin does not have ssl and crypto
      if [ $PHAML_OS = "cygwin" ]
      then
      PETSCLIBS="-L$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB"'/lib -lpetsc'
      fi

# PETSc before version 3.0.0 needed more include directories than new versions
#      PETSCINC='-I$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/include -I$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/bmake/$(PETSC_ARCH) -I$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB $MESSPASSINC" ;;

# for PETSC between 3.0.0 and 3.5
#      PETSCINC='-I$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/include -I$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB $MESSPASSINC"

# PETSc 3.6 moved finclude from include to include/petsc
      PETSCINC="-I$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB/include -I$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB/include/petsc -I$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB $MESSPASSINC"

      if [ $PHAML_PARALLEL = "sequential" -o $PHAML_PARALLEL = "openmp" ]
      then
# PETSc 3.6 moved the mpiuni from include to include/petsc
#         PETSCINC="$PETSCINC "'-I$(PETSC_HOME)/'"$PHAML_F90/$PHAML_PARLIB"'/include/mpiuni'
         PETSCINC="$PETSCINC -I$PETSC_HOME/$PHAML_F90/$PHAML_PARLIB/include/petsc/mpiuni"
      fi ;;

   no)
      PETSCLIBS=
      PETSCINC= ;;

   *) echo "mkmkfile.sh: $PHAML_PETSC is not a valid value for PHAML_PETSC"
      exit 1 ;;

esac

# SLEPc 3.9.1

case "$PHAML_SLEPC" in

   yes)
      SLEPCLIBS="-L$SLEPC_HOME/$PHAML_F90/$PHAML_PARLIB"'/lib -lslepc'
      SLEPCINC="-I$SLEPC_HOME/$PHAML_F90/$PHAML_PARLIB"'/include'
      if [ $PHAML_BLOPEX = "yes" ]
      then
         SLEPCLIBS="$SLEPCLIBS -lBLOPEX"
      fi

# SLEPc 3.2-p3 does not distinguish between calling the parpack routines
# and the arpack routines when SLEPc is compiled sequentially.  So I have
# parpack.dum.f90 call the arpack routines and compile it into the phaml
# library if ARPACK is included in a sequential build.  But, since
# libphaml is linked before libslepc, the linker still gets unresolved
# references to the parpack routines.  So I need to link libphaml again
# after libslepc.

      if [ $PHAML_ARPACK = "yes" ]
      then
         if [ $PHAML_PARALLEL = "sequential" -o $PHAML_PARALLEL = "openmp" ]
         then
            SLEPCLIBS="$SLEPCLIBS -lphaml"
         fi
      fi ;;

   no)
      SLEPCLIBS=
      SLEPCINC= ;;

   *) echo "mkmkfile.sh: $PHAML_SLEPC is not a valid value for PHAML_SLEPC"
      exit 1 ;;

esac

# hypre 2.10.0b(or newer?) from --download-hypre in PETSc

case "$PHAML_HYPRE" in

   yes)

      HYPRELIBS="-lHYPRE" ;;

   no)

      HYPRELIBS= ;;

   *) echo "mkmkfile.sh: $PHAML_HYPRE is not a valid value for PHAML_HYPRE"
      exit 1 ;;

esac

# ML 6.2 from --download-ml in PETSc

case "$PHAML_ML" in

   yes)

# With CentOS 7, ML needs libstdc++;  my login scripts set RELMAJOR on CentOS 7

      if [ $RELMAJOR = "7" ]
      then
         MLLIBS="-lml -lstdc++"
      else
         MLLIBS="-lml"
      fi ;;

   no)

      MLLIBS= ;;

   *) echo "mkmkfile.sh: $PHAML_ML is not a valid value for PHAML_ML"
      exit 1 ;;

esac

# MUMPS 4.10.0

case "$PHAML_MUMPS" in

   yes)

# For sequential PETSc, I used --with-mumps-dir and my own compiled MUMPS lib

      if [ $PHAML_PARALLEL = "sequential" -o $PHAML_PARALLEL = "openmp" ]
      then
#         MUMPSLIBS='-L$(MUMPS_HOME)/'"$PHAML_F90/$PHAML_PARLIB/lib -ldmumps -lmumps_common -lpord -lmpiseq"
# As of PETSc 3.7.2 I can use the --download-mumps approach
         MUMPSLIBS='-ldmumps -lmumps_common -lpord -lmpiseq'
      else

# For parallel PETSc, I used the --download approach to adding MUMPS support
# to PETSc.  This places these libraries in the PETSc library directory, and
# since these follow PETSc in the Makefile I don't need to give the directory
# here.  If you are using separately compiled MUMPS, SCALAPACK, BLACS and/or
# ParMETIS then you might need to specify some directory(s).

# if SUPERLU or ZOLTAN/PARMETIS is included, delay linking ParMETIS

# With PETSc 3.4.5 the BLACS were placed in libscalapack.a instead of their
# own library.  The commented out lines are what I used before that version.

         if [ $PHAML_SUPERLU = "yes" -o $PHAML_PARMETIS = "yes" ]
         then
#            MUMPSLIBS='-ldmumps -lmumps_common -lpord -lscalapack -lblacs'
            MUMPSLIBS='-ldmumps -lmumps_common -lpord -lscalapack'
         else
#            MUMPSLIBS='-ldmumps -lmumps_common -lpord -lscalapack -lblacs -lparmetis -lmetis'
            MUMPSLIBS='-ldmumps -lmumps_common -lpord -lscalapack -lparmetis -lmetis'
         fi

      fi ;;

   no)
      MUMPSLIBS= ;;

   *) echo "mkmkfile.sh: $PHAML_MUMPS is not a valid value for PHAML_MUMPS"
      exit 1 ;;

esac

# SuperLU installed with PETSc.  I used the --download approach.  If you
# compiled it separately, you might have to add directories here.

case "$PHAML_SUPERLU" in

   yes)

      if [ $PHAML_PARALLEL = "sequential" -o $PHAML_PARALLEL = "openmp" ]
      then
         SUPERLULIBS="-lsuperlu"

      else
# if ZOLTAN/PARMETIS is included, delay linking ParMETIS

         if [ $PHAML_PARMETIS = "yes" ]
         then
            SUPERLULIBS="-lsuperlu_dist"
         else
            SUPERLULIBS="-lsuperlu_dist -lparmetis -lmetis"
         fi

      fi ;;

   no)
      SUPERLULIBS= ;;

   *) echo "mkmkfile.sh: $PHAML_SUPERLU is not a valid value for PHAML_SUPERLU"
      exit 1 ;;

esac

# If there are any other libraries that you know you need to link in,
# define them here (or in step 3).

OTHERLIBS=

# end of Step 2

##############################################################################

# Step 3.
# Here you can override the values set in step 2 for particular system
# configurations, for example if a particular library is located in
# different directories on different machines.  The variable
# PHAML_SYSTEM determines which section of changes are used.  You should
# make a new section for each of your systems where you need to override
# the values of step 2.  Follow the examples below.  The intention is that
# the value for PHAML_SYSTEM be a machine name, but in reality you can use
# whatever character string you want.  You probably want to add that string
# to the help line, too.  Search for pepe to see where you might want
# to put your system names.

case "$PHAML_SYSTEM" in

########
# raritan is a frontend to a Linux cluster

   raritan)

# On raritan2, if I compile the executable on a node then it does not find
# lapack and blas even if I give the directory with -L, but is OK if I give
# the libraries explicitly.  (But it does find them when compiled on the
# frontend.)

         if [ $PHAML_BLAS = "standard" ]
         then
            BLASLIBS='/usr/lib64/libblas.so.3'
         fi
         if [ $PHAML_LAPACK = "standard" ]
         then
            LAPACKLIBS='/usr/lib64/liblapack.so.3'
         fi

# The X11 libraries Xaw and Xmu are missing on the nodes, or at least some of
# them, but I don't normally need them.  I might need to put them back if I
# compile phaml_graphics.

         XLIBS="-lXt -lXi -lX11 -lXext -lm"
      ;;

########
# pepe is a 64 bit quad core Linux laptop

   pepe)

# ARPACK needs etime, which can be obtained from libg2c
# That was before I used ARPACK with SLEPc; test if I still need this.
# Also, that line causes problems with my own gfortran 4.7.

#      if [ $PHAML_ARPACK = "yes" ]
#      then
#         OTHERLIBS="$OTHERLIBS -L/usr/lib/gcc/x86_64-redhat-linux/3.4.6 -lg2c"
#      fi;;

# Because I don't have the nagfor script in /usr/local/bin, I need to use Qpath.
# No longer true with nagfor 5.3.1-907

      if [ $PHAML_F90 = "nag" ]
      then
#         FFLAGS="$FFLAGS -Qpath /local/apps/nag/lib/NAG_Fortran"
#         LINKFLAGS="$LINKFLAGS -Qpath /local/apps/nag/lib/NAG_Fortran"

# NAG objects to -pthread and -m64 which are in mpif90 for OpenMPI and LAM.

# For OpenMPI, see what the compiler flags are in
# /usr/lib64/openmpi/share/openmpi/mpif90-wrapper-data.txt
# and set OMPI_FCFLAGS to them without -pthread -m64.
# If that file doesn't exist, use the flags shown by mpif90 --showme except for
# -pthread, -m64, -I flags, and the libraries.

# For LAM, don't use the wrappers and put all the other flags shown by
# --showme into the flags.  Also add two -W flags that pass -m64 to gcc.

         if [ $PHAML_PARLIB = "lam" ]
         then
            FFLAGS="$FFLAGS -I/usr/lib64/lam/include -I/usr/lib64/lam/include/64 -L/usr/lib64/lam/lib -llammpio -llamf77mpi -lmpi -llam -laio -laio -lutil -ldl -Wc,-m64 -Wl,-m64"
            LINKFLAGS="$LINKFLAGS -I/usr/lib64/lam/include -I/usr/lib64/lam/include/64 -L/usr/lib64/lam/lib -llammpio -llamf77mpi -lmpi -llam -laio -laio -lutil -ldl -Wc,-m64 -Wl,-m64"
            F90="nagfor"
            LINKER="nagfor"
         fi

# with the standard BLAS and LAPACK libraries, nagfor gives a warning
# about underflow having occured.

         if [ $PHAML_BLAS = "standard" -o $PHAML_LAPACK = "standard" ]
         then
            LINKFLAGS="$LINKFLAGS -no_underflow_warning"
         fi

# nagfor with PETSc fails to find some MPI routines, because the MPI libraries
# in LINKFLAGS occur before the PETSc libraries, so add LINKFLAGS to the end
# of OTHERLIBS.  Do this without a PETSc condition because it is possible it
# could happen without PETSc, too.
# 3/13/13 try without it

#         OTHERLIBS="$OTHERLIBS "'$(LINKFLAGS)'

      fi

# gfortran 4.7 is not finding it's own libquadmath

      if [ $PHAML_F90 = "gfortran" -a "x$GFORTRAN_HOME" == "x/local/apps/gfortran" ]
      then
         OTHERLIBS="$OTHERLIBS -L/local/apps/gfortran/lib64 -lquadmath"
      fi

# The Absoft compiler needs libU77 for flush and system

      if [ $PHAML_F90 = "absoft" ]
      then
         OTHERLIBS="$OTHERLIBS -lU77"
      fi;;

########
# looney is a Windows 7 computer with Cygwin and MinGW

   looney)

# Don't include X11 libraries

      NO_PHAML_GETS_GRAPHICSLIBS="yes"
      XLIBS="" ;;

########
# stampede is the TACC supercomputer consisting of KNL nodes

   stampede)

# Intel Fortran compiler flags

      if [ $PHAML_PARALLEL = "openmp" ]
      then
         FFLAGS="-w -O2 -no-vec -qopenmp -xMIC-AVX512"
      else
         FFLAGS="-w -O2 -no-vec -xMIC-AVX512"
      fi

# Use mkl for Lapack and BLAS; for now use the sequential version

      BLASLIBS=""
      LAPACKLIBS="-L/null"
      if [ $PHAML_PARALLEL = "openmp" ]
      then
         LINKFLAGS="-w -qopenmp -mkl=sequential -xMIC-AVX512"
      else
         LINKFLAGS="-w -mkl=sequential -xMIC-AVX512"
      fi ;;

########
# dirac is a 4 node KNL machine

   dirac)

# Intel Fortran compiler flags

      if [ $PHAML_PARALLEL = "openmp" -o $PHAML_PARALLEL = "hybrid_spawn" -o $PHAML_PARALLEL = "hybrid_nospawn" ]
      then
         FFLAGS="-w -O2 -no-vec -qopenmp -xMIC-AVX512"
      else
         FFLAGS="-w -O2 -no-vec -xMIC-AVX512"
      fi

# Use mkl for Lapack and BLAS; for now use the sequential version

      BLASLIBS=""
      LAPACKLIBS="-L/null"
      if [ $PHAML_PARALLEL = "openmp" -o $PHAML_PARALLEL = "hybrid_spawn" -o $PHAML_PARALLEL = "hybrid_nospawn" ]
      then
         LINKFLAGS="-w -qopenmp -mkl=sequential -xMIC-AVX512"
      else
         LINKFLAGS="-w -mkl=sequential -xMIC-AVX512"
      fi ;;

########
# suns is for our Sun workstations running solaris, which may have more than
# one Fortran 90 compiler

   suns)

# The Sun F90 needs some extra libraries
      if [ $PHAML_F90 = "sun" ]
      then
         OTHERLIBS="$OTHERLIBS -lnsl -lsocket"
      fi ;;

esac

# end of Step 3.  You should not need to change anything below here.

case "$PHAML_ZOLTAN" in

   no)
      ZOLTANMOD=
      ZOLTANLIBS= ;;

   yes)
      ZOLTANLIBS=$ZOLTANLIB
      if [ $PHAML_PARMETIS = "yes" ]
      then
         ZOLTANLIBS="$ZOLTANLIBS $PARMETISLIB"
      fi
      if [ $PHAML_DRUM = "yes" ]
      then
         ZOLTANLIBS="$DRUMLIB $ZOLTANLIBS"
      fi ;;

esac

if [ $PHAML_PARALLEL = "messpass_nospawn" -o $PHAML_PARALLEL = "hybrid_nospawn" ]
then
   case "$PHAML_GRAPHICS" in
      metro|mesa|opengl)
         PHAML_GETS_GRAPHICSLIBS='yes' ;;
   esac
fi

if [ $PHAML_PETSC == "yes" -a "x$NO_PHAML_GETS_GRAPHICSLIBS" != "xyes" ]
then
   PHAML_GETS_GRAPHICSLIBS='yes'
fi

##############################################################################

# You should not have to change anything below here.

##############################################################################

# export the shell variables needed by mkmkfile.sh in subdirectories

export PHAML_ARCH
export PHAML_OS
export PHAML_F90
export PHAML_C
export PHAML_HASHSIZE
export PHAML_PARALLEL
export PHAML_PARLIB
export PHAML_ELEMENT
export PHAML_GRAPHICS
export PHAML_BLAS
export PHAML_LAPACK
export PHAML_SLEPC
export PHAML_ARPACK
export PHAML_BLOPEX
export PHAML_PETSC
export PHAML_MUMPS
export PHAML_SUPERLU
export PHAML_HYPRE
export PHAML_ML
export PHAML_SYSTEM
export PHAML_ZOLTAN
export PHAML_PARMETIS
export PHAML_DRUM
export F90
export FFLAGS
export CC
export CFLAGS
export LINKER
export LINKFLAGS
export MPIF
export MODFLAG
export MODSUFFIX
export MAKELIB
export RANLIB
export PHAML_HOME
export MESSPASSLIBS
export OGLLIBS
export XLIBS
export LAPACKLIBS
export BLASLIBS
export ZOLTANLIBS
export SLEPCLIBS
export ARPACKLIBS
export PETSCLIBS
export MUMPSLIBS
export SUPERLULIBS
export HYPRELIBS
export MLLIBS
export OTHERLIBS
export PHAML_GETS_GRAPHICSLIBS
export ZOLTANMOD
export ZOLTANINC
export OGLMODS
export SLEPCINC
export PETSCINC
export MESSPASSINC
export RUNMPI

##############################################################################

# execute mkmkfile.sh for the src directory, testdir, and all the examples

echo "make makefile for src"
cd src
./mkmkfile.sh

case "$PHAML_ELEMENT" in
   triangle)
      echo "make makefile for testdir"
      cd ../testdir ;;
   tetrahedron|tetrahedron_fixdeg)
      echo "make makefile for testdir3D"
      cd ../testdir3D ;;
esac
./mkmkfile.sh

case "$PHAML_ELEMENT" in
   triangle)
      cd ../examples ;;
   tetrahedron|tetrahedron_fixdeg)
      cd ../examples3D ;;
esac
for dir in `ls`
do
   echo "make makefile for $dir"
   cd $dir
   ./mkmkfile.sh
   cd ..
done

cd ..

##############################################################################

# create the top level Makefile

if [ -f Makefile ]
then
   rm -f Makefile
fi
f=Makefile

echo "#---------------------------------------------------------------------!" 1>>$f
echo "#                                PHAML                                !" 1>>$f
echo "#                                                                     !" 1>>$f
echo "# The Parallel Hierarchical Adaptive MultiLevel code for solving      !" 1>>$f
echo "# linear elliptic partial differential equations of the form          !" 1>>$f
echo "# (PUx)x + (QUy)y + RU = F on 2D polygonal domains with mixed         !" 1>>$f
echo "# boundary conditions, and eigenvalue problems where F is lambda*U.   !" 1>>$f
echo "#                                                                     !" 1>>$f
echo "# PHAML is public domain software.  It was produced as part of work   !" 1>>$f
echo "# done by the U.S. Government, and is not subject to copyright in     !" 1>>$f
echo "# the United States.                                                  !" 1>>$f
echo "#                                                                     !" 1>>$f
echo "#     William F. Mitchell                                             !" 1>>$f
echo "#     Applied and Computational Mathematics Division                  !" 1>>$f
echo "#     National Institute of Standards and Technology                  !" 1>>$f
echo "#     william.mitchell@nist.gov                                       !" 1>>$f
echo "#     http://math.nist.gov/phaml                                      !" 1>>$f
echo "#                                                                     !" 1>>$f
echo "#---------------------------------------------------------------------!" 1>>$f
echo "" 1>>$f
echo "# Makefile created for system configuration:" 1>>$f
echo "#   Architecture:     " $PHAML_ARCH 1>>$f
echo "#   OS:               " $PHAML_OS 1>>$f
echo "#   F90 compiler:     " $PHAML_F90 1>>$f
echo "#   C compiler:       " $PHAML_C 1>>$f
echo "#   Hash size:        " $PHAML_HASHSIZE 1>>$f
echo "#   Parallel form:    " $PHAML_PARALLEL 1>>$f
echo "#   Parallel library: " $PHAML_PARLIB 1>>$f
echo "#   Element:          " $PHAML_ELEMENT 1>>$f
echo "#   Graphics:         " $PHAML_GRAPHICS 1>>$f
echo "#   BLAS:             " $PHAML_BLAS 1>>$f
echo "#   LAPACK:           " $PHAML_LAPACK 1>>$f
echo "#   SLEPc:            " $PHAML_SLEPC 1>>$f
echo "#   ARPACK:           " $PHAML_ARPACK 1>>$f
echo "#   BLOPEX:           " $PHAML_BLOPEX 1>>$f
echo "#   PETSc:            " $PHAML_PETSC 1>>$f
echo "#   MUMPS:            " $PHAML_MUMPS 1>>$f
echo "#   SUPERLU:          " $PHAML_SUPERLU 1>>$f
echo "#   hypre:            " $PHAML_HYPRE 1>>$f
echo "#   ML:               " $PHAML_ML 1>>$f
echo "#   Zoltan:           " $PHAML_ZOLTAN 1>>$f
echo "#   ParMETIS:         " $PHAML_PARMETIS 1>>$f
echo "#   DRUM:             " $PHAML_DRUM 1>>$f
echo "#   Specific system:  " $PHAML_SYSTEM 1>>$f
echo "" 1>>$f

echo "phaml:" 1>>$f
echo "	cd src; make lib" 1>>$f
echo "" 1>>$f

echo "clean:" 1>>$f
echo "	cd src; make clean" 1>>$f
case "$PHAML_ELEMENT" in
   triangle)
echo "	cd testdir; make clean" 1>>$f ;;
   tetrahedron|tetrahedron_fixdeg)
echo "	cd testdir3D; make clean" 1>>$f ;;
esac
case "$PHAML_ELEMENT" in
   triangle)
for dir in `ls examples`
do
echo "	cd examples/$dir; make clean" 1>>$f
done ;;
   tetrahedron|tetrahedron_fixdeg)
for dir in `ls examples3D`
do
echo "	cd examples3D/$dir; make clean" 1>>$f
done ;;
esac
echo "" 1>>$f

echo "reallyclean:" 1>>$f
echo "	cd src; make clean" 1>>$f
case "$PHAML_ELEMENT" in
   triangle)
echo "	cd testdir; make reallyclean" 1>>$f ;;
   tetrahedron|tetrahedron_fixdeg)
echo "	cd testdir3D; make reallyclean" 1>>$f ;;
esac
case "$PHAML_ELEMENT" in
   triangle)
for dir in `ls examples`
do
echo "	cd examples/$dir; make clean" 1>>$f
echo "	cd examples/$dir; rm -f Makefile Makefile.bak" 1>>$f
done ;;
   tetrahedron|tetrahedron_fixdeg)
for dir in `ls examples3D`
do
echo "	cd examples3D/$dir; make clean" 1>>$f
echo "	cd examples3D/$dir; rm -f Makefile Makefile.bak" 1>>$f
done ;;
esac
echo "	rm -f lib/*" 1>>$f
echo "	rm -f modules/*" 1>>$f
echo "	cd src; rm -f Makefile Makefile.bak" 1>>$f
echo "	cd testdir; rm -f Makefile Makefile.bak" 1>>$f
echo "	cd testdir3D; rm -f Makefile Makefile.bak" 1>>$f
echo "" 1>>$f

echo "test:" 1>>$f
case "$PHAML_ELEMENT" in
   triangle)
echo '	cd testdir; make test what=$(what)' 1>>$f ;;
   tetrahedron|tetrahedron_fixdeg)
echo '	cd testdir3D; make test what=$(what)' 1>>$f ;;
esac
echo "" 1>>$f
